#pragma once

#include "D3dRenderSystemExports.h"

#include "Direct3d/d3d10.h"
#include "Direct3d/d3dx10.h"

class D3dRenderTarget;
class D3dRenderSystem;

class D3DRENDERSYSTEM_API D3dBufferManager
{
friend class D3dRenderSystem;

public:
	typedef std::vector<ID3D10Buffer*> tBufferArray;
	typedef std::vector<ID3D10ShaderResourceView*> tSRVArray;

	D3dBufferManager(ID3D10Device& d3dDevice);
	~D3dBufferManager();

	bool Generate(Mesh& mesh, const D3dRenderSystem& renderSystem);
	void Delete(Mesh& mesh);
	
	bool Generate(Image& image, const D3dRenderSystem& renderSystem);
	void Delete(Image& image);

	bool Generate(D3dRenderTarget& renderTarget, const D3dRenderSystem& renderSystem);
	void Delete(D3dRenderTarget& renderTarget);

	const tBufferArray& GetBufferArray() const;
	const tSRVArray& GetSRVTextureArray() const;

private:
	struct VertexData
	{
		D3DXVECTOR3 position;
		D3DXVECTOR3 normal;
		D3DXVECTOR2 texCoord;
	};

	HRESULT CreateBuffer(D3D10_USAGE usage, UINT byteWidth, UINT bindFlags, UINT cpuAccessFlags, UINT miscFlags, const void* data);

private:
	static Logger logger;

	ID3D10Device* _d3dDevice;

#pragma warning(push)
#pragma warning(disable : 4251)
	tBufferArray _buffers;
	tSRVArray _srvTextures;
#pragma warning(pop)
};