#pragma once

#include "D3dRenderSystemExports.h"
#include "Common/Font.h"

#include "Direct3d/d3d10.h"
#include "Direct3d/d3dx10.h"

class D3dRenderSystem;

class D3DRENDERSYSTEM_API D3dFontManager
{
friend class D3dRenderSystem;
public:
	D3dFontManager(ID3D10Device& d3dDevice);
	~D3dFontManager();

	bool Generate(Font& font, const D3dRenderSystem& renderSystem);

	void Begin(Font& font);
	void End(Font& font);

	void SetFontColor(const tVec4f& color);
	void SetShadowColor(const tVec4f& color);
	void SetInsertionPos(const tVec2i& pos);
	void SetShadowed(bool enable);
	void SetLineHeight(unsigned int lineHeight);

	void DrawFormattedTextLine(Font& font, const char* text, ...);
	void DrawTextLine(Font& font, const char* text);

private:
	typedef std::vector<Font*> tFontArray;
	typedef std::vector<ID3DX10Font*> tD3dFontArray;
	typedef std::vector<ID3DX10Sprite*> tD3dSpriteArray;
	bool Initialize(const D3dRenderSystem& renderSystem);
	void DrawTextLine(Font& font, const char* text, const tVec2i& pos, const D3DXCOLOR& color);

private:
	static Logger logger;
	ID3D10Device* _d3dDevice;
	bool _shadowed;
	int _lineHeight;
#pragma warning(push)
#pragma warning(disable : 4251)
	tFontArray _fonts;
	tD3dFontArray _d3dFonts;
	tD3dSpriteArray _d3dSprites;
	tVec2i _pos;
	D3DXCOLOR _fontColor;
	D3DXCOLOR _shadowColor;
#pragma warning(pop)
};