#pragma once

#include "D3dRenderSystemExports.h"

#include "SceneGraph/LiSP.h"

#include "D3dRenderTarget.h"
#include "D3dEffect.h"

class D3dRenderSystem;

class D3DRENDERSYSTEM_API D3dShadowMapping : public Pass
{
private:
	class D3DRENDERSYSTEM_API GenDepthMapEffect : public D3dEffect
	{
	friend class D3dShadowMapping;
	friend class D3dRenderSystem;

	public:
		GenDepthMapEffect(const std::string& name);

		void UniformCallBack(D3dEffectManager& effectManager, const Material& material);
		void UniformCallBack(D3dEffectManager& effectManager, const Texture& texture, ID3D10ShaderResourceView& resource, float* texMtx);
		void UniformCallBack(D3dEffectManager& effectManager, const Transform& transform);

	private:
		bool Initialize(ID3D10Effect& effect);

	private:
		static const std::string GenDepthMapSourceFile;
		static const std::string NbrOfSplitsMacro;
		ID3D10EffectShaderResourceVariable* diffuseTex;
		ID3D10EffectMatrixVariable* world;
		ID3D10EffectMatrixVariable* view;
		ID3D10EffectMatrixVariable* projection;
		ID3D10EffectMatrixVariable* tex;
		ID3D10EffectScalarVariable* textured;
		ID3D10EffectVectorVariable* matDiffuse;
		ID3D10EffectMatrixVariable* viewProj;
		ID3D10EffectMatrixVariable* crop;
		ID3D10EffectMatrixVariable* liSP;
		ID3D10EffectScalarVariable* firstSplit;
		ID3D10EffectScalarVariable* lastSplit;
	};

public:
	class D3DRENDERSYSTEM_API DispDepthMapEffect : public D3dEffect
	{
	friend class D3dShadowMapping;

	public:
		DispDepthMapEffect(const std::string& name);

	private:
		bool Initialize(ID3D10Effect& effect);

	private:
		static const std::string DispDepthMapSourceFile;
		ID3D10EffectShaderResourceVariable* depthMap;
		ID3D10EffectShaderResourceVariable* depthMapArray;
		ID3D10EffectShaderResourceVariable* texMap;
		ID3D10EffectShaderResourceVariable* texMapArray;
		ID3D10EffectScalarVariable* arrayIndex;
		ID3D10EffectVectorVariable* mapSize;
	};

	class D3DRENDERSYSTEM_API BlurEffect : public D3dEffect
	{
	friend class D3dShadowMapping;

	public:
		BlurEffect(const std::string& name);

	private:
		bool Initialize(ID3D10Effect& effect);

	private:
		static const std::string BlurSourceFile;
		ID3D10EffectShaderResourceVariable* map;
		ID3D10EffectVectorVariable* size;
		ID3D10EffectVectorVariable* dim;
		ID3D10EffectScalarVariable* samples;
		ID3D10EffectMatrixVariable* tex;
	};

	enum FilterMode
	{
		None = 0,
		HWPCF,
		Box4x4PCF,
		Poisson25PCF,
		Box6x6PCF,
		Poisson32PCF,
		Box8x8PCF,
		Poisson64PCF,
		PCFPCSS,
		ExpSAVSM,
		ExpSAVSMPCSS
	};

	enum ErrDir
	{
		ZDir = 0,
		XDir,
		YDir,
		//Storage,
		ErrDirCOUNT
	};

	typedef std::vector<D3DXMATRIX> tMatrixArray;
	typedef std::vector<float> tPosArray;
	typedef std::vector<Frustum*> tFrustumArray;
	typedef std::vector<BoundingBox*> tBBArray;
	typedef std::vector<tVec2f> tDistArray;
	typedef std::vector<PolygonBody*> tBodyArray;
	typedef std::vector<D3DXVECTOR2> tVecArray;
	typedef std::vector<D3dRGFloatRT*> tMipmapChain;
	typedef std::vector<tVec4f> tUVArray;
	typedef std::vector< std::vector<tVec3f> > tErrValsArray;
	typedef std::vector<unsigned int> tSplitSmpleArray;

	D3dShadowMapping();
	~D3dShadowMapping();

	bool Initialize(D3dRenderSystem& renderSystem);
	void Render();

	const D3dRenderTarget& GetShadowMaps() const;
	float* GetNormLightViewProjMtx();
	const tMatrixArray& GetTexMatrixArray() const;
	float* GetSplitPositions();
	const tPosArray& GetSplitPos() const;
	float* GetLightViewMtx();
	float* GetLightProjMtx(unsigned int split);
	const tFrustumArray& GetSplitFrustums() const;
	tBBArray& GetSplitBBs();
	const tDistArray& GetNearAndFar() const;
	float GetAngleCamDirLightDir() const;
	const tBodyArray& GetBodyArray() const;	
	const tErrValsArray& GetErrorValues() const;
	const tDistArray& GetErrorRanges() const;

	GenDepthMapEffect& GetGenDepthMapEffect();

	unsigned int GetShadowMapSize() const;
	void SetShadowMapSize(unsigned int size);

	FilterMode GetFilterMode() const;
	void SetFilterMode(FilterMode mode);

	unsigned int GetNbrOfSplits() const;
	void SetNbrOfSplits(unsigned int nbrOfSplits);

	bool GetSwimmingRed() const;
	void SetSwimmingRed(bool enable);

	bool GetUseLiSP() const;
	void SetUseLiSP(bool enable);
	LiSP& GetLiSP();

	bool GetUseGS() const;
	void SetUseGS(bool enable);

	bool GetUseMinDepth() const;
	void SetUseMinDepth(bool enable);

	void SetDebug(bool enable);
	
	const Projection& GetLightProjection() const;
	void SetLightProjection(const Projection& proj);

	const View& GetLightView() const;
	void SetLightView(const View& view);

	Camera& GetCamera();
	void SetCamera(Camera& camera);

	float GetSplitParam() const;
	void SetSplitParam(float lambda);

	float GetBlurWidth() const;
	void SetBlurWidth(float width);
	
	bool GetAlignmentLSVS() const;
	void SetAlignmentLSVS(bool enable);

	float* GetSplitScales();
	float GetLightRadius() const;
	void SetLightRadius(float radius);

	bool GetUseFocusing() const;
	void SetUseFocusing(bool enable);

	float GetZNear() const;
	void SetZNear(float value);

	float GetPseudoNear() const;
	void SetPseudoNear(float value);

	float GetPseudoFar() const;
	void SetPseudoFar(float value);

	bool GetCalcError() const;
	void SetCalcError(bool enable);

	bool GetUseBBSel() const;
	void SetUseBBSel(bool enable);

	const tVec2f& GetDepthRange() const;

	const tSplitSmpleArray& GetSplitPosSamples() const;

	const DispDepthMapEffect& Get2dEffect() const;
	
	const tSplitSmpleArray& GetPseudoNFSamples() const;

	void UseScene(unsigned int id);
	void DisplayDepthMaps(const RenderSystem::Viewport& viewport);
	void DisplayMap(RenderSystem::Viewport& viewport, const D3dRenderTarget& rt);
	bool GenerateMinMaxMipmap();

	bool CheckRecompile(FilterMode mode) const;
	unsigned int GetFilterMacroVal(FilterMode mode) const;

private:
	void GenerateArrays();
	void DeleteArrays();
	void UpdateEyeViewProj();
	void UpdateLDirVDirAngle(const View& eView, const View& lView);
	void UpdatePointLightViewProj(const View& eView);
	void UpdateDirLightViewProj(const View& eView, const PolygonBody& body);
	bool GenerateRenderTargets();
	void DeleteRenderTargets();
	D3dRenderTarget& GetActiveRenderTarget();
	void AdjustCameraPlanes(const BoundingBox& aabb, const Frustum& frust, float& zNear, float& zFar);
	//void AdjustCameraPlanes(const BoundingBox& aabb, float& zNear, float& zFar);
	void CalcSplitPositions(float zNear, float zFar, unsigned int nbrOfSplits);
	void GenerateDepthMap(unsigned int split);
	void CalcPolygonBodyB(PolygonBody& bodyB, const Projection& lProj, const View& lView, const Frustum& cFrust, const Frustum* lFrust, BoundingBox& s);
	void CalcLightViewProjMat(unsigned int split, const PolygonBody& body, const tMat4f& lProjMat, const tMat4f& lViewMat);
	void GenerateDebugInfos(const tMat4f& invEyeViewProj, const PolygonBody& body, unsigned int split);
	void GenerateTexAtlasConstants(float unit, unsigned int factor);
	void FindCasters(unsigned int split);
	void Blur(D3dRenderTarget& src, D3dRenderTarget& dest, const tVec2f& dim, unsigned int split);
	float GetMinDepth();
	void DeleteMinMaxMipmap();
	void CalcErrorValues();
	
public:
	static const unsigned int MaxNbrOfSplits;

private:
	static const unsigned int NbrOfSamples;

	D3dRenderSystem* _renderSystem;
	
	GenDepthMapEffect _genSMEffect;
	DispDepthMapEffect _dispSMEffect;
	BlurEffect _blurSMEffect;

	View _lView;
	Projection* _lProj;
	Camera* _camera;
	LiSP* _liSP;

	D3dFloatDepthRT _depthMap;
	D3dRGBAFloatRT _vsmMap;
	D3dRGBAFloatRT _blur;
	D3dRGFloatCPURT _minmaxMIP;

	unsigned int _shadowMapSz;
	unsigned int _nbrOfSplits;
	unsigned int _sceneId;

	bool _useLiSP;
	bool _updateLightUp;
	bool _reduceSwimming;
	bool _useGS;
	bool _debug;
	bool _useMinDepth;
	FilterMode _filterMode;
	bool _focusing;
	bool _useBBSel;
	bool _calcError;

	float _gamma;
	float _angleCamDirLightDir;
	float _lambda;
	float _blurWidth;
	float _lightRadius;
	float _lightRadiusSplitCor;
	float _zNear;
	float _pseudoNear;
	float _pseudoFar;

#pragma warning(push)
#pragma warning(disable : 4251)
	static const D3DXMATRIX BiasMatrices[];
	static const tVec4f UV4[4];
	static const tVec4f UV9[9];
	static const tVec4f UV16[16];

	D3DXMATRIX _lViewMat;
	D3DXMATRIX _lProjMat;
	D3DXMATRIX _eViewMat;
	D3DXMATRIX _eProjMat;
	D3DXMATRIX _lSplitProjMat;
	D3DXMATRIX _lSplitViewMat;

	RenderSystem::tViewportArray _viewports;
	tMatrixArray _normLightViewProj;
	tMatrixArray _lightView;
	tMatrixArray _lightProj;
	tPosArray _splitPositions;
	tFrustumArray _splitFrustums;
	tBBArray _splitBBs;
	tBodyArray _intersectionBodys;
	tDistArray _planesNearFar;
	tMatrixArray _normMatrices;
	tMatrixArray _cropMatrices;
	tMatrixArray _liSPMatrices;
	tDistArray _filterScales;
	tVecArray _splitScales;
	tVec2f _depthRange;
	tMipmapChain _camDepthMipmap;
	tErrValsArray _errVals;
	tDistArray _errRanges;
	tUVArray _UV;
	tSplitSmpleArray _splitErrorSmples;
	tSplitSmpleArray _errPseudoNF;
#pragma warning(pop)

};