
#include "include/Common.fx"

//=============================================================================
// Uniforms
//=============================================================================

Texture2D depthMap;
Texture2D texMap;

Texture2DArray depthMapArray;
Texture2DArray texMapArray;

int g_arrayIndex;
int2 g_mapSize;

//=============================================================================
// Vertexshaders
//=============================================================================

PostProcess_VSOut PostProcess_VS(PostProcess_VSIn input)
{
	// Generate a full-screen triangle from vertex ID's

	PostProcess_VSOut output = (PostProcess_VSOut)0.0;

	output.tex = float2((input.vertexId << 1) & 2, input.vertexId & 2);
	output.pos = float4(output.tex * float2(2.0, -2.0) + float2(-1.0, 1.0), 0.0, 1.0);

	return output;
}

//=============================================================================
// Pixelshaders
//=============================================================================

float4 DisplayDepthMap_PS(PostProcess_PSIn input) : SV_Target
{
    float depth = depthMap.Sample(SampPointClamp, input.tex).r;

    return float4(depth, depth, depth, 1.0f);
}

float4 DisplayDepthMapArray_PS(PostProcess_PSIn input) : SV_Target
{
    float depth = depthMapArray.Sample(SampPointClamp, float3(input.tex, g_arrayIndex)).r;
    
    return float4(depth, depth, depth, 1.0f);
}

float4 DisplayVSMDepthMap_PS(PostProcess_PSIn input) : SV_Target
{
	float color = depthMap.Sample(SampPointClamp, input.tex).r;
	float depth = log(color) / ExpC;
	
	return float4(depth, depth, depth, 1.0f);
}

float4 DisplayMap_PS(PostProcess_PSIn input) : SV_Target
{
    return texMap.Sample(SampPointClamp, input.tex);
    //return (float4)texMap.Sample(SampPointClamp, input.tex).a / 1000.0;
}

float4 DisplayMapArray_PS(PostProcess_PSIn input) : SV_Target
{
    return texMapArray.Sample(SampPointClamp, float3(input.tex, g_arrayIndex));
}

float2 MinMaxMipMap_PS(PostProcess_PSIn input) : SV_Target
{
    float2 offset = 1.0f / g_mapSize;    
    
    float2 depth = depthMap.Sample(SampPointWrap, input.tex).rg, depth1;
    
    depth1 = depthMap.Sample(SampPointWrap, input.tex + float2(.0f, offset.y)).rg;
    depth = float2(min(depth.x, depth1.x), max(depth.y, depth1.y));
    
    depth1 = depthMap.Sample(SampPointWrap, input.tex + float2(offset.x, .0f)).rg;
    depth = float2(min(depth.x, depth1.x), max(depth.y, depth1.y));
    
    depth1 = depthMap.Sample(SampPointWrap, input.tex + float2(offset.x, offset.y)).rg;
    depth = float2(min(depth.x, depth1.x), max(depth.y, depth1.y));
    
    return depth;
}

float2 MinMipMap_PS(PostProcess_PSIn input) : SV_Target
{
    float2 offset = 1.0f / g_mapSize;    
    
    float depth = depthMap.Sample(SampPointWrap, input.tex).r, depth1;
    
    depth1 = depthMap.Sample(SampPointWrap, input.tex + float2(.0f, offset.y)).r;
    depth = min(depth, depth1);
    
    depth1 = depthMap.Sample(SampPointWrap, input.tex + float2(offset.x, .0f)).r;
    depth = min(depth, depth1);
    
    depth1 = depthMap.Sample(SampPointWrap, input.tex + float2(offset.x, offset.y)).r;
    depth = min(depth, depth1);
    
    return (float2)depth;
}

#include "include/Basic2d.fx"

//=============================================================================
// Techniques
//=============================================================================

technique10 DisplayDepthMap
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, PostProcess_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DisplayDepthMap_PS() ) );
        
        SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
        SetBlendState( BSNoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        //SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetRasterizerState( RSSolidFillCullFront );
    }  
}

technique10 DisplayDepthMapArray
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, PostProcess_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DisplayDepthMapArray_PS() ) );
        
        SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
        SetBlendState( BSNoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetRasterizerState( RSSolidFillCullFront );
    }  
}

technique10 DisplayVSMDepthMap
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, PostProcess_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DisplayVSMDepthMap_PS() ) );
        
        SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
        SetBlendState( BSNoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetRasterizerState( RSSolidFillCullFront );
    }  
}

technique10 DisplayTexMap
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, PostProcess_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DisplayMap_PS() ) );
        
        SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
        SetBlendState( BSNoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetRasterizerState( RSSolidFillCullFront );
    }  
}

technique10 DisplayTexMapArray
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, PostProcess_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, DisplayMapArray_PS() ) );
        
        SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
        SetBlendState( BSNoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        //SetBlendState( SrcAlphaBlendingAdd, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetRasterizerState( RSSolidFillCullFront );
    }  
}

technique10 GenMipmap
{
	pass GenMinMipmap
	{
		SetVertexShader( CompileShader( vs_4_0, PostProcess_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, MinMipMap_PS() ) );
        
        SetDepthStencilState( DSNoDepthStencil, 0x00000000 );
        SetBlendState( BSNoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetRasterizerState( RSSolidFillCullFront );	
	}
}