
#include "include/States.fx"

//=============================================================================
// Structures
//=============================================================================

struct PostProcess_VSIn
{
    uint vertexId : SV_VERTEXID;
};

struct PostProcess_VSOut
{
    float4 pos : SV_POSITION;
    float2 tex : TEXCOORD;
};

struct PostProcess_PSIn
{
    float4 pos : SV_POSITION;
    float2 tex : TEXCOORD;
};

//=============================================================================
// States
//=============================================================================

DepthStencilState DSPostProcess
{
    DepthEnable = false;
    DepthWriteMask = 0;
    StencilEnable = false;
};

RasterizerState RSPostProcess
{
    CullMode = None;
    FillMode = Solid;
    ScissorEnable = false;
    MultisampleEnable = false;
};


//=============================================================================
// Uniforms
//=============================================================================

cbuffer cbPerFrame
{
	float2	 g_SourceSize;            
	float2	 g_BlurDim;
	int		 g_BlurSamples;
	float4x4 g_mTex;
};

Texture2D map;

//=============================================================================
// Vertexshaders
//=============================================================================

PostProcess_VSOut PostProcess_VS(PostProcess_VSIn input)
{
	PostProcess_VSOut output = (PostProcess_VSOut)0.0;

	output.tex = float2((input.vertexId << 1) & 2, input.vertexId & 2);
	output.pos = float4(output.tex * float2(2.0, -2.0) + float2(-1.0, 1.0), 0.0, 1.0);

	return output;
}

PostProcess_VSOut PSPostProcess_VS(PostProcess_VSIn input)
{
	PostProcess_VSOut output = (PostProcess_VSOut)0.0;

	float2 uv  = float2((input.vertexId << 1) & 2, input.vertexId & 2);
	output.pos = float4(uv * float2(2.0, -2.0) + float2(-1.0, 1.0), 0.0, 1.0);
	output.tex = mul(output.pos, g_mTex).xy;

	return output;
}

//=============================================================================
// Pixelshaders
//=============================================================================

float4 BoxBlur_PS(PostProcess_PSIn input) : SV_Target
{
	float2 texelSize = 1 / g_SourceSize;
	float  blurSizeInv = 1 / float(g_BlurSamples);
	float2 sampleOffset = texelSize * g_BlurDim;
	float2 offset = 0.5f * float(g_BlurSamples - 1) * sampleOffset;

	float2 uv = input.tex - offset;

	float4 sum = (float4)0.0f;
	for (int i = 0; i < g_BlurSamples; ++i)
		sum += map.SampleLevel(SampBilinearClamp, uv + i * sampleOffset, 0);
	
	return sum * blurSizeInv;
}

//=============================================================================
// Techniques
//=============================================================================

technique10 BoxBlur
{
    pass p0
    {
        SetVertexShader(CompileShader(vs_4_0, PostProcess_VS()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, BoxBlur_PS()));
        
        SetDepthStencilState( DSPostProcess, 0x00000000 );
        SetBlendState( BSNoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetRasterizerState( RSPostProcess );
    }
    
    pass p1
    {
        SetVertexShader(CompileShader(vs_4_0, PSPostProcess_VS()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, BoxBlur_PS()));
        
        SetDepthStencilState( DSPostProcess, 0x00000000 );
        SetBlendState( BSNoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
        SetRasterizerState( RSPostProcess );
    }
}
