
#include "include/Shadows.fx"
#include "include/PSShadows.fx"

// disable warning 4121: gradient based operation inside of flow control
#pragma warning( disable : 4121 )

//=======================================================================================
// Uniforms
//=======================================================================================

cbuffer cbPerFrame
{
	float4x4 g_mView;
    float4x4 g_mProj;
    float4x4 g_mCamView;
    float4x4 g_mLightView[NUMSPLITS];
    float4x4 g_mNormLightViewProj[NUMSPLITS];
    float	 g_Splits[NUMSPLITS];
    bool	 g_visSplits;
	bool	 g_visTexels;
	int		 g_shadowMapSz;
	float2	 g_LightRadius[NUMSPLITS];
	int		 g_texelSz;
	float3	 g_CamPos;
	bool	 g_visSampling;
	bool	 g_renderShadows;
	int		 g_errorDir;
	float3	 g_viewVec;
	float3	 g_lightVec;
	bool	 g_hwPCF;
	bool	 g_blockerSearch;
};

Texture2D texShadow;

//=======================================================================================
// Vertexshaders
//=======================================================================================

VSOut VSRenderPSSMmain(VSIn input)
{
    VSOut output = (VSOut)0.0;

    float4 worldPos = mul( float4( input.pos, 1 ), g_mWorld );
    float4 eyePos = mul( worldPos, g_mView );
    
    output.pos   = mul( eyePos, g_mProj );
    output.tex   = mul( float4( input.tex, 1, 1 ), g_mDiffuseTex ).xy;
	output.wPos  = worldPos.xyz;
    output.wNorm = normalize( mul( input.norm, (float3x3)g_mWorld ) );
	output.depth = -mul( worldPos, g_mCamView ).z;
    
	return output;
}

//=======================================================================================
// Pixelshaders
//=======================================================================================

float4 PSRenderPSSMmain(PSIn input, uniform int method) : SV_Target
{   
    float3 normal = normalize( input.wNorm );
    float3 viewVec = normalize( g_CamPos - input.wPos );
    float3 lightVec = normalize( g_light.position.xyz - input.wPos );
    
    float3 shadowContrib = (float3)1.0;
    float2 uv = (float2)0.0;
    uint split = 0;
    
	for (split = 0; split < NUMSPLITS; split++)
	{
		if (input.depth < g_Splits[split])
		{
			float4 lPos = mul( float4(input.wPos, 1), g_mNormLightViewProj[split] );
			uv = lPos.xy / lPos.w;
			float zReceiver = lPos.z / lPos.w;
			
			if (g_renderShadows)
			{
				float zEye = .0f;
				switch (method)
				{
					case 0:
						shadowContrib = ShadowLookUp(texShadow, uv, zReceiver);
						break;
					case 1:
						shadowContrib = HWPCFShadowLookUp(texShadow, uv, zReceiver);
						break;
					case 2:
						//zEye = mul(float4(input.wPos, 1), g_mLightView[split]).z;
						//shadowContrib = BoxPCFLookUp(texShadow, uv, zReceiver, zEye, g_LightRadius[split], g_hwPCF, g_blockerSearch);
						shadowContrib = BoxPCF(texShadow, uv, zReceiver, g_shadowMapSz, g_hwPCF);
						break;
					case 3:
						//zEye = mul(float4(input.wPos, 1), g_mLightView[split]).z;
						//shadowContrib = PoissonPCFLookUp(texShadow, uv, zReceiver, zEye, g_LightRadius[split], g_hwPCF, g_blockerSearch);
						shadowContrib = PoissonPCF(texShadow, uv, zReceiver, g_shadowMapSz, g_hwPCF);
						break;
					case 4:
						zEye = mul(float4(input.wPos, 1), g_mLightView[split]).z;
						shadowContrib = PoissonPCFPCSSLookUp(texShadow, uv, zReceiver, zEye, g_LightRadius[split], g_hwPCF);
						break;
					//case 3:
					//	shadowContrib = ExpVSMLookUp(texShadow, uv, zReceiver, MinVariance);						
					//	break;
				}
			}						
			break;
		}
	}
    
    bool inside = false;
    for (uint i = 0; i < NUMSPLITS; i++)
		inside |= (split == i && uv.x >= UV[i].x && uv.y >= UV[i].y && uv.x <= UV[i].z && uv.y <= UV[i].w);
    
    if (!inside)
		shadowContrib = 1.0f;
    
    //shadowContrib = max(0.1f, shadowContrib);
    float4 color = Lighting(viewVec, normal, lightVec, input.tex, shadowContrib);
    
    if (inside)
    {
		[flatten] if (g_visSplits)
    		color.rgb = lerp(color.rgb, SplitColors[split%4], 0.2f);

		[flatten] if (g_visTexels)
			color *= VisualizeTexels(uv, g_shadowMapSz, g_texelSz, FACTOR);
	
		[flatten] if (g_visSampling)
			color *= VisualizeSampling(uv, g_shadowMapSz, FACTOR, g_errorDir);
	}
	
    return color;
}

float4 PSRenderPSSMmainBBSel(PSIn input, uniform int method) : SV_Target
{
	float3 normal = normalize( input.wNorm );
    float3 viewVec = normalize( g_CamPos - input.wPos );
    float3 lightVec = normalize( g_light.position.xyz - input.wPos );
    
    float3 shadowContrib = (float3)1.0;
    float2 uv = (float2)0.0;
    uint split = 0;
    float epsilon = 0.0f;
    
    if (method != 0) epsilon = 0.002f;
    
    for (split = 0; split < NUMSPLITS; split++)
    {
		float4 lPos = mul( float4(input.wPos, 1), g_mNormLightViewProj[split] );
		uv = lPos.xy / lPos.w;
		float zReceiver = lPos.z / lPos.w;
		
		if ((uv.x >= UV[split].x + epsilon && uv.y >= UV[split].y + epsilon && zReceiver >= 0.0f) &&
			(uv.x <= UV[split].z - epsilon && uv.y <= UV[split].w - epsilon && zReceiver <= 1.0f))
		{
			if (g_renderShadows)
			{
				float zEye = .0f;
				switch (method)
				{
					case 0:	
						shadowContrib = ShadowLookUp(texShadow, uv, zReceiver);
						break;
					case 1:
						shadowContrib = HWPCFShadowLookUp(texShadow, uv, zReceiver);
						break;
					case 2:
						//zEye = mul(float4(input.wPos, 1), g_mLightView[split]).z;
						//shadowContrib = BoxPCFLookUp(texShadow, uv, zReceiver, zEye, g_LightRadius[split], g_hwPCF, g_blockerSearch);
						shadowContrib = BoxPCF(texShadow, uv, zReceiver, g_shadowMapSz, g_hwPCF);
						break;
					case 3:
						//zEye = mul(float4(input.wPos, 1), g_mLightView[split]).z;
						//shadowContrib = PoissonPCFLookUp(texShadow, uv, zReceiver, zEye, g_LightRadius[split], g_hwPCF, g_blockerSearch);
						shadowContrib = PoissonPCF(texShadow, uv, zReceiver, g_shadowMapSz, g_hwPCF);
						break;
					case 4:
						zEye = mul(float4(input.wPos, 1), g_mLightView[split]).z;						
						shadowContrib = PoissonPCFPCSSLookUp(texShadow, uv, zReceiver, zEye, g_LightRadius[split], g_hwPCF);
						break;
					//case 3:
					//	shadowContrib = ExpVSMLookUp(texShadow, uv, zReceiver, MinVariance);
					//	break;								
				}						
			}
			break;
		}
	}
    
    bool inside = false;
    for (uint i = 0; i < NUMSPLITS; i++)
		inside |= (split == i && uv.x >= UV[i].x && uv.y >= UV[i].y && uv.x <= UV[i].z && uv.y <= UV[i].w);
    
    if (!inside)
		shadowContrib = 1.0f;
    
    //shadowContrib = max(0.1f, shadowContrib);    
    float4 color = Lighting(viewVec, normal, lightVec, input.tex, shadowContrib);
    
    if (inside)
    {
		[flatten] if (g_visSplits)
    		color.rgb = lerp(color.rgb, SplitColors[split%4], 0.2f);

		[flatten] if (g_visTexels)
			color *= VisualizeTexels(uv, g_shadowMapSz, g_texelSz, FACTOR);
	
		[flatten] if (g_visSampling)
			color *= VisualizeSampling(uv, g_shadowMapSz, FACTOR, g_errorDir);
	}
		
    return color;
}

#include "include/Basic3d.fx"

//=======================================================================================
// Techniques
//=======================================================================================

technique10 RenderPSSM
{
	pass Common
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmain(0) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
    
	pass HWPCF
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmain(1) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	pass BoxPCF
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmain(2) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	pass PoissonPCF
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmain(3) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	pass PCSSPCF
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmain(4) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	//pass ExpSAVSM
	//{
		//SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		//SetGeometryShader( NULL );
		//SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmain(3) ) );
	    //
		//SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		//SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		//SetRasterizerState( RSSolidFillCullBack );
	//}
}

technique10 RenderPSSMBBSel
{
	pass Common
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmainBBSel(0) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	pass HWPCF
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmainBBSel(1) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	pass BoxPCF
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmainBBSel(2) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	pass PoissonPCF
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmainBBSel(3) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	pass PCSSPCF
	{
		SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		SetGeometryShader( NULL );
		SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmainBBSel(4) ) );
	    
		SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		SetBlendState( BSAlphaToCoverage, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetRasterizerState( RSSolidFillCullBack );
	}
	
	//pass ExpSAVSM
	//{
		//SetVertexShader( CompileShader( vs_4_0, VSRenderPSSMmain() ) );
		//SetGeometryShader( NULL );
		//SetPixelShader( CompileShader( ps_4_0, PSRenderPSSMmainBBSel(3) ) );
	    //
		//SetDepthStencilState( DSEnableDepthLessEqual, 0x00000000 );
		//SetBlendState( NoBlending, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		//SetRasterizerState( RSSolidFillCullBack );
	//}	
}

